from edc_constants.constants import DONT_KNOW, NO, NONE, OTHER
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_dx_review.utils import raise_if_clinical_review_does_not_exist
from edc_form_validators import FormValidator


class HealthEconomicsHouseholdHeadFormValidator(
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        self.applicable_if(NO, field="hoh", field_applicable="relationship_to_hoh")
        self.validate_other_specify(field="relationship_to_hoh")
        self.validate_other_specify(field="hoh_religion")
        self.validate_other_specify(field="hoh_ethnicity")
        self.validate_other_specify(field="hoh_education")
        self.validate_other_specify(field="hoh_marital_status")
        self.m2m_single_selection_if(DONT_KNOW, NONE, m2m_field="hoh_insurance")
        self.m2m_other_specify(
            OTHER, m2m_field="hoh_insurance", field_other="hoh_insurance_other"
        )



from __future__ import annotations

from datetime import date, datetime

from dateutil.relativedelta import relativedelta
from django.apps import apps as django_apps
from edc_consent.utils import get_consent_for_period_or_raise
from edc_constants.constants import MALE, NO, YES
from edc_form_validators import INVALID_ERROR, FormValidator
from edc_model import InvalidFormat, duration_to_date

INVALID_DURATION_IN_CARE = "invalid_duration_in_care"


class SubjectScreeningFormValidator(FormValidator):
    def clean(self):
        if not self.patient_log_identifier:
            self.raise_validation_error("Select a Patient log", error_code=INVALID_ERROR)
        self.get_consent_for_period_or_raise()

        if (
            self.cleaned_data.get("consent_ability")
            and self.cleaned_data.get("consent_ability") == NO
        ):
            self.raise_validation_error(
                {
                    "consent_ability": (
                        "You may NOT screen this subject without their verbal consent."
                    )
                },
                INVALID_ERROR,
            )

        self.required_if(YES, field="in_care_6m", field_required="in_care_duration")
        self.duration_in_care_is_6m_or_more_or_raise()

        self.validate_hiv_section()
        self.validate_dm_section()
        self.validate_htn_section()

        self.not_applicable_if(
            MALE, field="gender", field_applicable="pregnant", inverse=False
        )

        self.validate_suitability_for_study()

    @property
    def report_datetime(self):
        return self.cleaned_data.get("report_datetime")

    def get_consent_for_period_or_raise(self):
        return get_consent_for_period_or_raise(self.report_datetime)

    def duration_in_care_is_6m_or_more_or_raise(self, fieldname: str = None) -> None:
        dt: date | datetime | None = None
        fieldname = fieldname or "in_care_duration"
        in_care_duration = self.cleaned_data.get(fieldname)
        report_datetime = self.cleaned_data.get("report_datetime")
        if report_datetime and in_care_duration:
            try:
                dt = duration_to_date(in_care_duration, report_datetime)
            except InvalidFormat as e:
                self.raise_validation_error({fieldname: f"Invalid format. {e}"}, INVALID_ERROR)
            if dt + relativedelta(months=6) > report_datetime:
                self.raise_validation_error(
                    {fieldname: "Expected at least 6m from the report date"},
                    INVALID_DURATION_IN_CARE,
                )

    def validate_hiv_section(self):
        self.applicable_if(YES, field="hiv_dx", field_applicable="hiv_dx_6m")
        self.required_if(YES, field="hiv_dx_6m", field_required="hiv_dx_ago")
        self.duration_in_care_is_6m_or_more_or_raise("hiv_dx_ago")
        self.applicable_if(YES, field="hiv_dx", field_applicable="art_unchanged_3m")
        self.applicable_if(YES, field="hiv_dx", field_applicable="art_stable")
        self.applicable_if(YES, field="hiv_dx", field_applicable="art_adherent")

    def validate_dm_section(self):
        self.applicable_if(YES, field="dm_dx", field_applicable="dm_dx_6m")
        self.required_if(YES, field="dm_dx_6m", field_required="dm_dx_ago")
        self.duration_in_care_is_6m_or_more_or_raise("dm_dx_ago")
        self.applicable_if(YES, field="dm_dx", field_applicable="dm_complications")

    def validate_htn_section(self):
        self.applicable_if(YES, field="htn_dx", field_applicable="htn_dx_6m")
        self.required_if(YES, field="htn_dx_6m", field_required="htn_dx_ago")
        self.duration_in_care_is_6m_or_more_or_raise("htn_dx_ago")
        self.applicable_if(YES, field="htn_dx", field_applicable="htn_complications")

    def validate_suitability_for_study(self):
        self.required_if(
            YES, field="unsuitable_for_study", field_required="reasons_unsuitable"
        )
        self.applicable_if(
            YES, field="unsuitable_for_study", field_applicable="unsuitable_agreed"
        )
        if self.cleaned_data.get("unsuitable_agreed") == NO:
            self.raise_validation_error(
                {
                    "unsuitable_agreed": "The study coordinator MUST agree "
                    "with your assessment. Please discuss before continuing."
                },
                INVALID_ERROR,
            )

    @property
    def patient_log_identifier(self):
        return (
            self.cleaned_data.get("patient_log_identifier")
            or self.instance.patient_log_identifier
        )

    @property
    def patient_log_model_cls(self):
        return django_apps.get_model("intecomm_screening.patientlog")



from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_form_validators import FormValidator
from edc_he.constants import ALL_WINDOWS_SCREENED, SOME_WINDOWS_SCREENED


class HealthEconomicsAssetsFormValidator(
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        self.validate_other_specify(field="water_source")
        self.validate_other_specify(field="toilet")
        self.validate_other_specify(field="roof_material")
        self.validate_other_specify(field="external_wall_material")
        self.validate_other_specify(field="external_window_material")
        self.applicable_if(
            ALL_WINDOWS_SCREENED,
            SOME_WINDOWS_SCREENED,
            field="window_screens",
            field_applicable="window_screen_type",
        )
        self.validate_other_specify(field="floor_material")
        self.validate_other_specify(field="light_source")
        self.validate_other_specify(field="cooking_fuel")



from django.urls import reverse
from django.utils.html import format_html
from edc_constants.constants import COMPLETE, YES
from edc_form_validators import INVALID_ERROR, FormValidator

INVALID_RANDOMIZE = "INVALID_RANDOMIZE"
INVALID_COMPLETE_GROUP_FIRST = "INVALID_COMPLETE_GROUP_FIRST"
INVALID_STATUS_NOT_COMPLETE = "INVALID_STATUS_NOT_COMPLETE"


class PatientGroupRandoFormValidator(FormValidator):
    def clean(self):
        self.block_changes_if_already_randomized()
        if not self.cleaned_data.get("name"):
            raise self.raise_validation_error(
                {"name": "This field is required"}, INVALID_ERROR
            )
        if not self.instance.id:
            raise self.raise_validation_error(
                "Complete Patient Group form first", INVALID_COMPLETE_GROUP_FIRST
            )
        if not self.instance.status == COMPLETE:
            url = reverse(
                "intecomm_screening_admin:intecomm_screening_patientgroup_change",
                args=(self.instance.id,),
            )
            raise self.raise_validation_error(
                format_html(
                    f'Return to the <A href="{url}">'
                    "Patient Group form</A> and verify the status is set to COMPLETE"
                ),
                INVALID_STATUS_NOT_COMPLETE,
            )
        if (
            self.cleaned_data.get("randomize_now") == YES
            and self.cleaned_data.get("confirm_randomize_now") != "RANDOMIZE"
        ):
            self.raise_validation_error(
                {
                    "confirm_randomize_now": (
                        "If you wish to randomize this group, please confirm"
                    )
                },
                INVALID_RANDOMIZE,
            )
        self.not_required_if_true(
            self.cleaned_data.get("randomize_now") != YES,
            field="confirm_randomize_now",
            msg="Only complete if you are ready to randomize now.",
        )

    def block_changes_if_already_randomized(self):
        if self.instance.randomized:
            self.raise_validation_error(
                "A randomized group may not be changed", INVALID_RANDOMIZE
            )



from edc_constants.constants import YES
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_form_validators import FormValidator


class HealthEconomicsPropertyFormValidator(
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        self.applicable_if(YES, field="land_owner", field_applicable="land_value_known")
        self.required_if(YES, field="land_value_known", field_required="land_value")
        self.applicable_if(
            YES, field="land_additional", field_applicable="land_additional_known"
        )
        self.required_if(
            YES, field="land_additional_known", field_required="land_additional_value"
        )
